from django.contrib import admin

from openbook_communities.models import (
    CollaborationTool,
    Community,
    CommunityMembership,
    Task,
    ExternalLink,
)


@admin.register(Community)
class CommunityAdmin(admin.ModelAdmin):
    readonly_fields = ["name"]
    list_display = ("title", "creator", "created")
    search_fields = ("title", "creator__email")

    def has_add_permission(self, request, obj=None):
        return True  # needs to be true for adding communities via /admin (needed for creating posts)

    def save_model(self, request, obj, form, change):
        super().save_model(request, obj, form, change)
        if not change:
            CommunityMembership.create_membership(
                user=obj.creator, is_administrator=True, is_moderator=False, community=obj
            )


@admin.register(CommunityMembership)
class CommunityMembershipAdmin(admin.ModelAdmin):
    list_display = ("community", "user")
    search_fields = ("community__title", "community__creator__email")


@admin.register(Task)
class CommunityTaskAdmin(admin.ModelAdmin):
    model = Task
    list_display = (
        "community",
        "name",
        "description",
        "visibility",
        "location_type",
        "location",
        "geolocation",
        "creator",
        "created",
        "contact_email",
        "contact_phone",
        "thumbnail",
        "regularity",
        "slots_taken",
        "slots_available",
    )


@admin.register(ExternalLink)
class CommunityLinkAdmin(admin.ModelAdmin):
    list_display = ("community", "type", "url")


@admin.register(CollaborationTool)
class CommunityCollaborationToolAdmin(admin.ModelAdmin):
    list_display = ("url", "domain", "title", "collection_type", "description")
